module Locmare
  module ListGroupModule
    class Base
      attr_accessor :list_group_name, :list_name, :operators, :options, :item_name, 
        :controller_manifest, :action_manifest, :list_manifest, :model, :table_name,
        :page_status, :item_wheres, :list_wheres, :wheres
      
      def initialize list_group_name, list_name, operators, options = {}
        @list_group_name = list_group_name
        @list_name = list_name
        @operators = operators
        @options = options
        
        @list_group_manifest = LocalManifest.manifest.list_groups[@list_group_name]
        @list_manifest = @list_group_manifest.lists[@list_name]
        @wheres = @list_manifest.wheres
        @includes = @list_manifest.includes
        
        @controller_name = @list_group_name
        @action_name = @list_name
        @controller_manifest = ::Manifest.manifest.controllers[@controller_name]
        @action_manifest = @controller_manifest.actions[@action_name]
        @item_name = @action_manifest.item_name
        @model = ::Manifest.item_name_to_model @item_name
        @table_name = @model.table_name
        self.init
      end
      
      def init
      end
      
      def reset
        @page_status = nil
        @item = nil
        @total = nil
      end
      
      def page_status
        @page_status ||= LibModule::PageStatus.load(self, self.total, @options)
      end
      
      def model_name
        @model.model_name
      end
      
      def limited?
        self.max_page_size > 0
      end
      
      def unlimited?
        !self.limited?
      end
      
      def default_page_size
        @action_manifest.default_page_size
      end
      
      def max_page_size
        @action_manifest.max_page_size
      end
      
      def order
        order = @options['order'] || @options[:order] || @action_manifest.order
        order = 'updated_at' unless ::Manifest.manifest.models[@item_name].attributes[order]
        @table_name + '.' + order
      end
      
      def direction
        direction = @options['direction'] || @options[:direction] || @action_manifest.direction
        if direction < 0
          'desc'
        elsif direction > 0
          'asc'
        else
          ''
        end
      end
      
      def filter_id
        @options['id'] || @options[:id]
      end
      
      def order_by
        self.order + ' ' + self.direction
      end
      
      def include_hash
        @includes.includes
      end
      
      def arel
        r = @model
        return @model.__send__('find_' + @action_name)
      end
      
      def items
        self.total  # get total
        return [] unless self.page_status  #.pager
        @items ||= self.arel.order(
          self.order_by
        ).offset(self.page_status.offset).limit(self.page_status.limit)
        self.boost
        @items
      end
      
      def total
        @total ||= self.arel.count
      end
      
      def boost_manifests
        @model.my_peta.boost.select do |boost_name, boost_manifest|
          boost_manifest.level == 'show'
        end
      end
      
      def boost
        manifests = self.boost_manifests
        @items.each do |item|
          manifests.each do |boost_manifest|
            item.boost_manifest # ?
          end
        end
      end
      
      def private_list_where
        owner_model = if @model.element?
          @model.root_model
        else
          @model
        end
        case owner_model.owner_type
        when :author
          operator = @operators.author
        when :artist
          operator = @operators.artist
        end
        [owner_model.table_name + '.' + owner_model.owner_type.to_s + '_id = ?', operator.id]
      end
      
    end
    
  end
end
