# -*- encoding: utf-8 -*-
require 'spec_helper'
#システム画像
describe SystemPicture do
  include ManifestsMacros
  before do
    @model = SystemPicture
  end
  
  describe 'ペタナイズに於いて' do
    it '管理者が投稿して管理する画像てある。よって、ペタナイズのタイプはシステムリソースである' do
      expect(@model.ancestors.include?(Peta::SystemResource)).to eq true
    end
    it '親モデルは持たない' do
      expect(@model.my_peta.parent_model_name).to be_blank
    end
    describe 'ブーストに於いて' do
      it '特筆すべき注意事項はない' do
        expect(@model.my_peta.boost.keys).to be_empty
      end
    end
    
  end
  
  describe 'コントローラに於いて' do
    before do
      @controller = @model.my_controller
    end
    it 'スクロールのためのコントローラである' do
      expect(@model.item_name).to eq 'system_picture'
    end
    describe 'actionに於いて' do
      before do
        @actions = @controller.actions
        @actions_names = ['index', 'show', 
          'count']
      end
      it 'これらのアクションが定義されている' do
        set_actions? @actions, @actions_names
      end
      describe 'baseに於いて' do
      end
      describe 'indexに於いて' do
        before do
          @action = @actions['index']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          # スクロールの公開リストを返すように定義している
          set_list_action? @action, 'system_picture', 'public'
        end
      end
      describe 'showに於いて' do
        before do
          @action = @actions['show']
        end
        it 'showタイプのアクションである' do
          expect(@action.type).to eq 'show'
        end
      end
      describe 'countに於いて' do
        before do
          @action = @actions['count']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          #スクロールの公開リストのカウントを返すように定義している
          set_list_action? @action, 'system_picture', 'public'
        end
      end
    end
    
  end
  
  describe 'モデルに於いて' do
    before do
      @my_manifest = @model.my_manifest
    end
    it 'サポート機能が動作している' do
      expect(@my_manifest.model_name).to eq 'system_picture'
      expect(@my_manifest.classify).to eq @model
      expect(@my_manifest.table_name).to eq 'system_pictures'
    end
    
    describe 'associationsに於いて' do
      describe 'belongs_toに於いて' do
        before do
          @belongs_to = @my_manifest.associations.belongs_to
          @names = []
        end
        it 'これらのモデルに所属している' do
          set_associations? @belongs_to, @names
        end
      end
      describe 'has_manyに於いて' do
        before do
          @has_many = @model.my_manifest.associations.has_many
          @names = ['balloons', 'speech_balloon_templates', 'licenses', 'writing_formats']
        end
        it 'これらのモデルを所持している' do
          set_associations? @has_many, @names
        end
        describe 'balloonsに於いて' do
          before do
            @mani = @has_many['balloons']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名, 外部キーと中間モデルが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'balloon'
            expect(@mani.foreign_key).to eq 'system_picture_id'
            expect(@mani.through).to be nil
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている, 中間モデルフラグ
            expect(@mani.through?).to be false
            expect(@mani.model).to eq Balloon
          end
        end
        describe 'speech_balloon_templatesに於いて' do
          before do
            @mani = @has_many['speech_balloon_templates']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名, 外部キーと中間モデルが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'speech_balloon_template'
            expect(@mani.foreign_key).to eq 'system_picture_id'
            expect(@mani.through).to be nil
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている, 中間モデルフラグ
            expect(@mani.through?).to be false
            expect(@mani.model).to eq SpeechBalloonTemplate
          end
        end
        describe 'licensesに於いて' do
          before do
            @mani = @has_many['licenses']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名, 外部キーと中間モデルが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'license'
            expect(@mani.foreign_key).to eq 'system_picture_id'
            expect(@mani.through).to be nil
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている, 中間モデルフラグ
            expect(@mani.through?).to be false
            expect(@mani.model).to eq License
          end
        end
        describe 'writing_formatsに於いて' do
          before do
            @mani = @has_many['writing_formats']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名, 外部キーと中間モデルが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'writing_format'
            expect(@mani.foreign_key).to eq 'system_picture_id'
            expect(@mani.through).to be nil
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている, 中間モデルフラグ
            expect(@mani.through?).to be false
            expect(@mani.model).to eq WritingFormat
          end
        end
      end
    end
    
    describe 'attributesに於いて' do
      before do
        @attributes = @model.my_manifest.attributes
      end
      it 'これらのカラムを定義している' do
        set_attributes? @attributes, 
          ['id', 'ext', 'width', 'height', 'filesize', 'md5', 
          'created_at', 'updated_at']
      end
      describe 'extに於いて' do
        before do
          @attribute = @attributes['ext']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'widthに於いて' do
        before do
          @attribute = @attributes['width']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'heightに於いて' do
        before do
          @attribute = @attributes['height']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'filesizeに於いて' do
        before do
          @attribute = @attributes['filesize']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'md5に於いて' do
        before do
          @attribute = @attributes['md5']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
    end
    
  end
  
end
